@@ -1,3 +1,6 @@
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 from datetime import datetime
 import re
 import traceback
@@ -8,8 +11,8 @@ import pandas as pd
 from collections import defaultdict
 from pandas.tslib import OutOfBoundsDatetime
 
-from .core import indexing, ops, utils
-from .core.formatting import format_timestamp, first_n_items
+from .core import duck_array_ops, indexing, ops, utils
+from .core.formatting import format_timestamp, first_n_items, last_item
 from .core.variable import as_variable, Variable
 from .core.pycompat import iteritems, OrderedDict, PY3, basestring
 
@@ -142,6 +145,12 @@ def decode_cf_datetime(num_dates, units, calendar=None):
             # strings, in which case we fall back to using netCDF4
             raise OutOfBoundsDatetime
 
+        # fixes: https://github.com/pydata/pandas/issues/14068
+        # these lines check if the the lowest or the highest value in dates
+        # cause an OutOfBoundsDatetime (Overflow) error
+        pd.to_timedelta(flat_num_dates.min(), delta) + ref_date
+        pd.to_timedelta(flat_num_dates.max(), delta) + ref_date
+
         dates = (pd.to_timedelta(flat_num_dates, delta) + ref_date).values
 
     except (OutOfBoundsDatetime, OverflowError):
@@ -369,10 +378,13 @@ class DecodedCFDatetimeArray(utils.NDArrayMixin):
         self.array = array
         self.units = units
         self.calendar = calendar
-        # Verify at least one date can be decoded successfully.
-        # Otherwise, tracebacks end up swallowed by Dataset.__repr__ when users
-        # try to view their lazily decoded array.
-        example_value = first_n_items(array, 1) or 0
+
+        # Verify that at least the first and last date can be decoded
+        # successfully. Otherwise, tracebacks end up swallowed by
+        # Dataset.__repr__ when users try to view their lazily decoded array.
+        example_value = np.concatenate([first_n_items(array, 1) or [0],
+                                        last_item(array) or [0]])
+
         try:
             result = decode_cf_datetime(example_value, units, calendar)
         except Exception:
@@ -612,7 +624,7 @@ def maybe_encode_dtype(var, name=None):
     if 'dtype' in var.encoding:
         dims, data, attrs, encoding = _var_as_tuple(var)
         dtype = np.dtype(encoding.pop('dtype'))
-        if dtype != var.dtype and dtype.kind != 'O':
+        if dtype != var.dtype:
             if np.issubdtype(dtype, np.integer):
                 if (np.issubdtype(var.dtype, np.floating) and
                         '_FillValue' not in var.attrs):
@@ -620,12 +632,20 @@ def maybe_encode_dtype(var, name=None):
                                   'point data as an integer dtype without '
                                   'any _FillValue to use for NaNs' % name,
                                   RuntimeWarning, stacklevel=3)
-                data = ops.around(data)[...]
+                data = duck_array_ops.around(data)[...]
             if dtype == 'S1' and data.dtype != 'S1':
                 data = string_to_char(np.asarray(data, 'S'))
                 dims = dims + ('string%s' % data.shape[-1],)
             data = data.astype(dtype=dtype)
-            var = Variable(dims, data, attrs, encoding)
+        var = Variable(dims, data, attrs, encoding)
+    return var
+
+
+def maybe_default_fill_value(var):
+    # make NaN the fill value for float types:
+    if ('_FillValue' not in var.attrs and
+       np.issubdtype(var.dtype, np.floating)):
+        var.attrs['_FillValue'] = np.nan
     return var
 
 
@@ -712,6 +732,7 @@ def encode_cf_variable(var, needs_copy=True, name=None):
     var, needs_copy = maybe_encode_offset_and_scale(var, needs_copy)
     var, needs_copy = maybe_encode_fill_value(var, needs_copy)
     var = maybe_encode_dtype(var, name)
+    var = maybe_default_fill_value(var)
     var = maybe_encode_bools(var)
     var = ensure_dtype_not_object(var)
     return var
@@ -901,7 +922,7 @@ def decode_cf(obj, concat_characters=True, mask_and_scale=True,
         identify coordinates.
     drop_variables: string or iterable, optional
         A variable or list of variables to exclude from being parsed from the
-        dataset.This may be useful to drop variables with problems or
+        dataset. This may be useful to drop variables with problems or
         inconsistent values.
 
     Returns
@@ -916,10 +937,12 @@ def decode_cf(obj, concat_characters=True, mask_and_scale=True,
         attrs = obj.attrs
         extra_coords = set(obj.coords)
         file_obj = obj._file_obj
+        encoding = obj.encoding
     elif isinstance(obj, AbstractDataStore):
         vars, attrs = obj.load()
         extra_coords = set()
         file_obj = obj
+        encoding = obj.get_encoding()
     else:
         raise TypeError('can only decode Dataset or DataStore objects')
 
@@ -927,8 +950,10 @@ def decode_cf(obj, concat_characters=True, mask_and_scale=True,
         vars, attrs, concat_characters, mask_and_scale, decode_times,
         decode_coords, drop_variables=drop_variables)
     ds = Dataset(vars, attrs=attrs)
-    ds = ds.set_coords(coord_names.union(extra_coords))
+    ds = ds.set_coords(coord_names.union(extra_coords).intersection(vars))
     ds._file_obj = file_obj
+    ds.encoding = encoding
+
     return ds
 
 
