@@ -16,8 +16,7 @@ import pandas as pd
 from pandas.tslib import OutOfBoundsDatetime
 
 from .options import OPTIONS
-from .pycompat import (
-    PY2, unicode_type, bytes_type, dask_array_type, OrderedDict, basestring)
+from .pycompat import PY2, unicode_type, bytes_type, dask_array_type
 
 
 def pretty_print(x, numchars):
@@ -49,7 +48,7 @@ def ensure_valid_repr(string):
     On Python 2, this means we need to convert unicode to bytes. We won't need
     this function once we drop Python 2.7 support.
     """
-    if PY2 and isinstance(string, unicode):
+    if PY2 and isinstance(string, unicode_type):
         string = string.encode('utf-8')
     return string
 
@@ -254,8 +253,15 @@ def summarize_coord(name, var, col_width):
 
 
 def summarize_attr(key, value, col_width=None):
-    # ignore col_width for now to more clearly distinguish attributes
-    return u'    %s: %s' % (key, maybe_truncate(value))
+    """Summary for __repr__ - use ``X.attrs[key]`` for full value."""
+    # Indent key and add ':', then right-pad if col_width is not None
+    k_str = u'    %s:' % key
+    if col_width is not None:
+        k_str = pretty_print(k_str, col_width)
+    # Replace tabs and newlines, so we print on one line in known width
+    v_str = unicode_type(value).replace(u'\t', u'\\t').replace(u'\n', u'\\n')
+    # Finally, truncate to the desired display width
+    return maybe_truncate(u'%s %s' % (k_str, v_str), OPTIONS['display_width'])
 
 
 EMPTY_REPR = u'    *empty*'
@@ -327,7 +333,7 @@ def unindexed_dims_repr(dims, coords):
     unindexed_dims = [d for d in dims if d not in coords]
     if unindexed_dims:
         dims_str = u', '.join(u'%s' % d for d in unindexed_dims)
-        return u'Unindexed dimensions:\n' + u' ' * 4 + dims_str
+        return u'Dimensions without coordinates: ' + dims_str
     else:
         return None
 
@@ -399,7 +405,8 @@ def dataset_repr(ds):
     dims_start = pretty_print(u'Dimensions:', col_width)
     summary.append(u'%s(%s)' % (dims_start, dim_summary(ds)))
 
-    summary.append(coords_repr(ds.coords, col_width=col_width))
+    if ds.coords:
+        summary.append(coords_repr(ds.coords, col_width=col_width))
 
     unindexed_dims_str = unindexed_dims_repr(ds.dims, ds.coords)
     if unindexed_dims_str:
