@@ -16,6 +16,8 @@ import inspect
 
 import numpy as np
 import pandas as pd
+from datetime import datetime
+import pytest
 
 from xarray import DataArray
 
@@ -28,6 +30,8 @@ from xarray.plot.utils import (_determine_cmap_params,
 from . import TestCase, requires_matplotlib
 
 
+@pytest.mark.flaky
+@pytest.mark.skip(reason='maybe flaky')
 def text_in_fig():
     '''
     Return the set of all text in the figure
@@ -73,6 +77,7 @@ class PlotTestCase(TestCase):
         plotmethod(ax=axes[0])
         self.assertTrue(axes[0].has_data())
 
+    @pytest.mark.slow
     def imshow_called(self, plotmethod):
         plotmethod()
         images = plt.gca().findobj(mpl.image.AxesImage)
@@ -99,6 +104,7 @@ class TestPlot(PlotTestCase):
     def test2d_uniform_calls_imshow(self):
         self.assertTrue(self.imshow_called(self.darray[:, :, 0].plot.imshow))
 
+    @pytest.mark.slow
     def test2d_nonuniform_calls_contourf(self):
         a = self.darray[:, :, 0]
         a.coords['dim_1'] = [2, 1, 89]
@@ -114,8 +120,9 @@ class TestPlot(PlotTestCase):
         self.assertArrayEqual([-0.5, 0.5, 1.5], _infer_interval_breaks([0, 1]))
         self.assertArrayEqual([-0.5, 0.5, 5.0, 9.5, 10.5],
                               _infer_interval_breaks([0, 1, 9, 10]))
-        self.assertArrayEqual(pd.date_range('20000101', periods=4) - np.timedelta64(12, 'h'),
-                              _infer_interval_breaks(pd.date_range('20000101', periods=3)))
+        self.assertArrayEqual(
+            pd.date_range('20000101', periods=4) - np.timedelta64(12, 'h'),
+            _infer_interval_breaks(pd.date_range('20000101', periods=3)))
 
         # make a bounded 2D array that we will center and re-infer
         xref, yref = np.meshgrid(np.arange(6), np.arange(5))
@@ -138,6 +145,7 @@ class TestPlot(PlotTestCase):
         ax = plt.gca()
         self.assertTrue(ax.has_data())
 
+    @pytest.mark.slow
     def test_convenient_facetgrid(self):
         a = easy_array((10, 15, 4))
         d = DataArray(a, dims=['y', 'x', 'z'])
@@ -154,6 +162,7 @@ class TestPlot(PlotTestCase):
         with self.assertRaisesRegexp(ValueError, '[Ff]acet'):
             d[0].plot(x='x', y='y', col='z', ax=plt.gca())
 
+    @pytest.mark.slow
     def test_subplot_kws(self):
         a = easy_array((10, 15, 4))
         d = DataArray(a, dims=['y', 'x', 'z'])
@@ -168,6 +177,7 @@ class TestPlot(PlotTestCase):
             except AttributeError:
                 self.assertEqual(ax.get_axis_bgcolor(), 'r')
 
+    @pytest.mark.slow
     def test_plot_size(self):
         self.darray[:, 0, 0].plot(figsize=(13, 5))
         assert tuple(plt.gcf().get_size_inches()) == (13, 5)
@@ -193,6 +203,7 @@ class TestPlot(PlotTestCase):
         with self.assertRaisesRegexp(ValueError, 'cannot provide `aspect`'):
             self.darray.plot(aspect=1)
 
+    @pytest.mark.slow
     def test_convenient_facetgrid_4d(self):
         a = easy_array((10, 15, 2, 3))
         d = DataArray(a, dims=['y', 'x', 'columns', 'rows'])
@@ -247,6 +258,7 @@ class TestPlot1D(PlotTestCase):
         p = self.darray.plot.line()
         self.assertTrue(isinstance(p[0], mpl.lines.Line2D))
 
+    @pytest.mark.slow
     def test_plot_nans(self):
         self.darray[1] = np.nan
         self.darray.plot.line()
@@ -298,6 +310,7 @@ class TestPlotHistogram(PlotTestCase):
         h = self.darray.plot.hist()
         self.assertTrue(isinstance(h[-1][0], mpl.patches.Rectangle))
 
+    @pytest.mark.slow
     def test_plot_nans(self):
         self.darray[0, 0, 0] = np.nan
         self.darray.plot.hist()
@@ -326,6 +339,7 @@ class TestDetermineCmapParams(TestCase):
         self.assertIsNone(cmap_params['levels'])
         self.assertIsNone(cmap_params['norm'])
 
+    @pytest.mark.slow
     def test_integer_levels(self):
         data = self.data + 1
 
@@ -465,11 +479,13 @@ class TestDiscreteColorMap(TestCase):
         self.data_min = distance.min()
         self.data_max = distance.max()
 
+    @pytest.mark.slow
     def test_recover_from_seaborn_jet_exception(self):
         pal = _color_palette('jet', 4)
         self.assertTrue(type(pal) == np.ndarray)
         self.assertEqual(len(pal), 4)
 
+    @pytest.mark.slow
     def test_build_discrete_cmap(self):
         for (cmap, levels, extend, filled) in [('jet', [0, 1], 'both', False),
                                                ('hot', [-4, 4], 'max', True)]:
@@ -485,6 +501,7 @@ class TestDiscreteColorMap(TestCase):
             else:
                 self.assertEqual(ncmap.colorbar_extend, 'max')
 
+    @pytest.mark.slow
     def test_discrete_colormap_list_of_levels(self):
         for extend, levels in [('max', [-1, 2, 4, 8, 10]),
                                ('both', [2, 5, 10, 11]),
@@ -501,6 +518,7 @@ class TestDiscreteColorMap(TestCase):
                     self.assertEqual('max', primitive.cmap.colorbar_extend)
                 self.assertEqual(len(levels) - 1, len(primitive.cmap.colors))
 
+    @pytest.mark.slow
     def test_discrete_colormap_int_levels(self):
         for extend, levels, vmin, vmax in [('neither', 7, None, None),
                                            ('neither', 7, None, 20),
@@ -636,17 +654,27 @@ class Common2dMixin:
         self.assertEqual('x', ax.get_ylabel())
 
     def test_positional_coord_string(self):
-        with self.assertRaisesRegexp(ValueError, 'cannot supply only one'):
-            self.plotmethod('y')
-        with self.assertRaisesRegexp(ValueError, 'cannot supply only one'):
-            self.plotmethod(y='x')
+        self.plotmethod(y='x')
+        ax = plt.gca()
+        self.assertEqual('x', ax.get_ylabel())
+        self.assertEqual('y', ax.get_xlabel())
+
+        self.plotmethod(x='x')
+        ax = plt.gca()
+        self.assertEqual('x', ax.get_xlabel())
+        self.assertEqual('y', ax.get_ylabel())
 
     def test_bad_x_string_exception(self):
-        with self.assertRaisesRegexp(ValueError, 'x and y must be coordinate'):
+        with self.assertRaisesRegexp(
+                ValueError, 'x and y must be coordinate variables'):
             self.plotmethod('not_a_real_dim', 'y')
+        with self.assertRaisesRegexp(
+                ValueError, 'x must be a dimension name if y is not supplied'):
+            self.plotmethod(x='not_a_real_dim')
+        with self.assertRaisesRegexp(
+                ValueError, 'y must be a dimension name if x is not supplied'):
+            self.plotmethod(y='not_a_real_dim')
         self.darray.coords['z'] = 100
-        with self.assertRaisesRegexp(ValueError, 'cannot supply only one'):
-            self.plotmethod('z')
 
     def test_coord_strings(self):
         # 1d coords (same as dims)
@@ -797,15 +825,17 @@ class Common2dMixin:
         d = DataArray(data, dims=['x', 'y', 'time'])
         fg = d.plot.pcolormesh(col='time')
         # check that all color limits are the same
-        self.assertTrue(len(set(m.get_clim() for m in fg._mappables)) == 1)
+        self.assertEqual(len(set(m.get_clim() for m in fg._mappables)), 1)
         # check that all colormaps are the same
-        self.assertTrue(len(set(m.get_cmap().name for m in fg._mappables)) == 1)
+        self.assertEqual(len(set(m.get_cmap().name for m in fg._mappables)), 1)
 
 
+@pytest.mark.slow
 class TestContourf(Common2dMixin, PlotTestCase):
 
     plotfunc = staticmethod(xplt.contourf)
 
+    @pytest.mark.slow
     def test_contourf_called(self):
         # Having both statements ensures the test works properly
         self.assertFalse(self.contourf_called(self.darray.plot.imshow))
@@ -815,6 +845,7 @@ class TestContourf(Common2dMixin, PlotTestCase):
         artist = self.plotmethod()
         self.assertTrue(isinstance(artist, mpl.contour.QuadContourSet))
 
+    @pytest.mark.slow
     def test_extend(self):
         artist = self.plotmethod()
         self.assertEqual(artist.extend, 'neither')
@@ -832,6 +863,7 @@ class TestContourf(Common2dMixin, PlotTestCase):
         artist = self.plotmethod(vmin=-10, vmax=0)
         self.assertEqual(artist.extend, 'max')
 
+    @pytest.mark.slow
     def test_2d_coord_names(self):
         self.plotmethod(x='x2d', y='y2d')
         # make sure labels came out ok
@@ -839,6 +871,7 @@ class TestContourf(Common2dMixin, PlotTestCase):
         self.assertEqual('x2d', ax.get_xlabel())
         self.assertEqual('y2d', ax.get_ylabel())
 
+    @pytest.mark.slow
     def test_levels(self):
         artist = self.plotmethod(levels=[-0.5, -0.4, 0.1])
         self.assertEqual(artist.extend, 'both')
@@ -847,6 +880,7 @@ class TestContourf(Common2dMixin, PlotTestCase):
         self.assertEqual(artist.extend, 'neither')
 
 
+@pytest.mark.slow
 class TestContour(Common2dMixin, PlotTestCase):
 
     plotfunc = staticmethod(xplt.contour)
@@ -876,7 +910,7 @@ class TestContour(Common2dMixin, PlotTestCase):
             (1.0, 1.0, 1.0))
         # the last color is now under "over"
         self.assertEqual(
-             _color_as_tuple(artist.cmap._rgba_over),
+            _color_as_tuple(artist.cmap._rgba_over),
             (0.0, 0.0, 1.0))
 
     def test_cmap_and_color_both(self):
@@ -887,6 +921,7 @@ class TestContour(Common2dMixin, PlotTestCase):
         with self.assertRaises(Exception):
             self.plotmethod(cmap=['k', 'b'])
 
+    @pytest.mark.slow
     def test_2d_coord_names(self):
         self.plotmethod(x='x2d', y='y2d')
         # make sure labels came out ok
@@ -913,6 +948,7 @@ class TestPcolormesh(Common2dMixin, PlotTestCase):
         artist = self.plotmethod()
         self.assertEqual(artist.get_array().size, self.darray.size)
 
+    @pytest.mark.slow
     def test_2d_coord_names(self):
         self.plotmethod(x='x2d', y='y2d')
         # make sure labels came out ok
@@ -931,10 +967,12 @@ class TestPcolormesh(Common2dMixin, PlotTestCase):
         self.assertTrue(artist.get_array().size <= self.darray.size)
 
 
+@pytest.mark.slow
 class TestImshow(Common2dMixin, PlotTestCase):
 
     plotfunc = staticmethod(xplt.imshow)
 
+    @pytest.mark.slow
     def test_imshow_called(self):
         # Having both statements ensures the test works properly
         self.assertFalse(self.imshow_called(self.darray.plot.contourf))
@@ -949,6 +987,7 @@ class TestImshow(Common2dMixin, PlotTestCase):
         self.darray.plot.imshow()
         self.assertEqual('auto', plt.gca().get_aspect())
 
+    @pytest.mark.slow
     def test_cannot_change_mpl_aspect(self):
 
         with self.assertRaisesRegexp(ValueError, 'not available in xarray'):
@@ -959,13 +998,15 @@ class TestImshow(Common2dMixin, PlotTestCase):
         self.assertEqual('auto', plt.gca().get_aspect())
         assert tuple(plt.gcf().get_size_inches()) == (10, 5)
 
+    @pytest.mark.slow
     def test_primitive_artist_returned(self):
         artist = self.plotmethod()
         self.assertTrue(isinstance(artist, mpl.image.AxesImage))
 
+    @pytest.mark.slow
     def test_seaborn_palette_needs_levels(self):
         try:
-            import seaborn
+            import seaborn  # noqa
             with self.assertRaises(ValueError):
                 self.plotmethod(cmap='husl')
         except ImportError:
@@ -975,6 +1016,7 @@ class TestImshow(Common2dMixin, PlotTestCase):
         with self.assertRaisesRegexp(ValueError, 'requires 1D coordinates'):
             self.plotmethod(x='x2d', y='y2d')
 
+
 class TestFacetGrid(PlotTestCase):
 
     def setUp(self):
@@ -983,6 +1025,7 @@ class TestFacetGrid(PlotTestCase):
                                 coords={'z': ['a', 'b', 'c']})
         self.g = xplt.FacetGrid(self.darray, col='z')
 
+    @pytest.mark.slow
     def test_no_args(self):
         self.g.map_dataarray(xplt.contourf, 'x', 'y')
 
@@ -997,6 +1040,7 @@ class TestFacetGrid(PlotTestCase):
             fontsize = ax.title.get_size()
             self.assertLessEqual(fontsize, 12)
 
+    @pytest.mark.slow
     def test_names_appear_somewhere(self):
         self.darray.name = 'testvar'
         self.g.map_dataarray(xplt.contourf, 'x', 'y')
@@ -1008,6 +1052,7 @@ class TestFacetGrid(PlotTestCase):
         for label in ['x', 'y']:
             self.assertIn(label, alltxt)
 
+    @pytest.mark.slow
     def test_text_not_super_long(self):
         self.darray.coords['z'] = [100 * letter for letter in 'abc']
         g = xplt.FacetGrid(self.darray, col='z')
@@ -1019,6 +1064,7 @@ class TestFacetGrid(PlotTestCase):
         t0 = g.axes[0, 0].get_title()
         self.assertTrue(t0.endswith('...'))
 
+    @pytest.mark.slow
     def test_colorbar(self):
         vmin = self.darray.values.min()
         vmax = self.darray.values.max()
@@ -1032,6 +1078,7 @@ class TestFacetGrid(PlotTestCase):
 
         self.assertEqual(1, len(find_possible_colorbars()))
 
+    @pytest.mark.slow
     def test_empty_cell(self):
         g = xplt.FacetGrid(self.darray, col='z', col_wrap=2)
         g.map_dataarray(xplt.imshow, 'x', 'y')
@@ -1040,10 +1087,12 @@ class TestFacetGrid(PlotTestCase):
         self.assertFalse(bottomright.has_data())
         self.assertFalse(bottomright.get_visible())
 
+    @pytest.mark.slow
     def test_norow_nocol_error(self):
         with self.assertRaisesRegexp(ValueError, r'[Rr]ow'):
             xplt.FacetGrid(self.darray)
 
+    @pytest.mark.slow
     def test_groups(self):
         self.g.map_dataarray(xplt.imshow, 'x', 'y')
         upperleft_dict = self.g.name_dicts[0, 0]
@@ -1052,16 +1101,19 @@ class TestFacetGrid(PlotTestCase):
 
         self.assertDataArrayEqual(upperleft_array, z0)
 
+    @pytest.mark.slow
     def test_float_index(self):
         self.darray.coords['z'] = [0.1, 0.2, 0.4]
         g = xplt.FacetGrid(self.darray, col='z')
         g.map_dataarray(xplt.imshow, 'x', 'y')
 
+    @pytest.mark.slow
     def test_nonunique_index_error(self):
         self.darray.coords['z'] = [0.1, 0.2, 0.2]
         with self.assertRaisesRegexp(ValueError, r'[Uu]nique'):
             xplt.FacetGrid(self.darray, col='z')
 
+    @pytest.mark.slow
     def test_robust(self):
         z = np.zeros((20, 20, 2))
         darray = DataArray(z, dims=['y', 'x', 'z'])
@@ -1083,6 +1135,7 @@ class TestFacetGrid(PlotTestCase):
         largest = max(abs(x) for x in numbers)
         self.assertLess(largest, 21)
 
+    @pytest.mark.slow
     def test_can_set_vmin_vmax(self):
         vmin, vmax = 50.0, 1000.0
         expected = np.array((vmin, vmax))
@@ -1092,12 +1145,14 @@ class TestFacetGrid(PlotTestCase):
             clim = np.array(image.get_clim())
             self.assertTrue(np.allclose(expected, clim))
 
+    @pytest.mark.slow
     def test_can_set_norm(self):
         norm = mpl.colors.SymLogNorm(0.1)
         self.g.map_dataarray(xplt.imshow, 'x', 'y', norm=norm)
         for image in plt.gcf().findobj(mpl.image.AxesImage):
             self.assertIs(image.norm, norm)
 
+    @pytest.mark.slow
     def test_figure_size(self):
 
         self.assertArrayEqual(self.g.fig.get_size_inches(), (10, 3))
@@ -1120,6 +1175,7 @@ class TestFacetGrid(PlotTestCase):
         with self.assertRaisesRegexp(ValueError, "Can't use"):
             g = xplt.plot(self.darray, row=2, col='z', ax=plt.gca(), size=6)
 
+    @pytest.mark.slow
     def test_num_ticks(self):
         nticks = 99
         maxticks = nticks + 1
@@ -1134,10 +1190,12 @@ class TestFacetGrid(PlotTestCase):
             self.assertGreaterEqual(xticks, nticks / 2.0)
             self.assertGreaterEqual(yticks, nticks / 2.0)
 
+    @pytest.mark.slow
     def test_map(self):
         self.g.map(plt.contourf, 'x', 'y', Ellipsis)
         self.g.map(lambda: None)
 
+    @pytest.mark.slow
     def test_map_dataset(self):
         g = xplt.FacetGrid(self.darray.to_dataset(name='foo'), col='z')
         g.map(plt.contourf, 'x', 'y', 'foo')
@@ -1156,6 +1214,7 @@ class TestFacetGrid(PlotTestCase):
         self.assertIn('colors!', text_in_fig())
         self.assertEqual(1, len(find_possible_colorbars()))
 
+    @pytest.mark.slow
     def test_set_axis_labels(self):
         g = self.g.map_dataarray(xplt.contourf, 'x', 'y')
         g.set_axis_labels('longitude', 'latitude')
@@ -1163,6 +1222,7 @@ class TestFacetGrid(PlotTestCase):
         for label in ['longitude', 'latitude']:
             self.assertIn(label, alltxt)
 
+    @pytest.mark.slow
     def test_facetgrid_colorbar(self):
         a = easy_array((10, 15, 4))
         d = DataArray(a, dims=['y', 'x', 'z'], name='foo')
@@ -1176,13 +1236,14 @@ class TestFacetGrid(PlotTestCase):
         d.plot.imshow(x='x', y='y', col='z', add_colorbar=False)
         self.assertEqual(0, len(find_possible_colorbars()))
 
+    @pytest.mark.slow
     def test_facetgrid_polar(self):
         # test if polar projection in FacetGrid does not raise an exception
         self.darray.plot.pcolormesh(col='z',
                                     subplot_kws=dict(projection='polar'),
                                     sharex=False, sharey=False)
 
-        
+
 class TestFacetGrid4d(PlotTestCase):
 
     def setUp(self):
@@ -1195,6 +1256,7 @@ class TestFacetGrid4d(PlotTestCase):
 
         self.darray = darray
 
+    @pytest.mark.slow
     def test_default_labels(self):
         g = xplt.FacetGrid(self.darray, col='col', row='row')
         self.assertEqual((2, 3), g.axes.shape)
@@ -1208,3 +1270,22 @@ class TestFacetGrid4d(PlotTestCase):
         # Top row should be labeled
         for label, ax in zip(self.darray.coords['col'].values, g.axes[0, :]):
             self.assertTrue(substring_in_axes(label, ax))
+
+
+class TestDatetimePlot(PlotTestCase):
+
+    def setUp(self):
+        '''
+        Create a DataArray with a time-axis that contains datetime objects.
+        '''
+        month = np.arange(1, 13, 1)
+        data = np.sin(2 * np.pi * month / 12.0)
+
+        darray = DataArray(data, dims=['time'])
+        darray.coords['time'] = np.array([datetime(2017, m, 1) for m in month])
+
+        self.darray = darray
+
+    def test_datetime_line_plot(self):
+        # test if line plot raises no Exception
+        self.darray.plot.line()
